@@ -21,7 +21,7 @@ from ethtx.models.semantics_model import AddressSemantics, ERC20Semantics
 
 
 class AddressInfo(BaseModel):
-    address: str
+    address: Optional[str]
     name: str
     badge: Optional[str]
 
@@ -37,7 +37,6 @@ class DecodedTransactionMetadata(BaseModel):
     receiver: Optional[AddressInfo]
     tx_index: int
     tx_value: int
-    eth_price: Optional[float]
     gas_limit: int
     gas_used: int
     success: bool
@@ -54,9 +53,9 @@ class DecodedEvent(BaseModel):
     tx_hash: str
     timestamp: datetime
     contract: AddressInfo
-    index: int
+    index: Optional[int]
     call_id: Optional[str]
-    event_signature: str
+    event_signature: Optional[str]
     event_name: str
     parameters: List[Argument]
     event_guessed: bool = False
